#!/usr/bin/env python
import serial
import subprocess
import time
import redis
import pika
import os
import datetime
import logging
import logging.handlers
import argparse
import ConfigParser
import sys

config = ConfigParser.ConfigParser()
config.read("/etc/piscan/piscan.ini")

LOG_FILENAME = config.get("system", "logloc") + "/pirec.log"
LOG_LEVEL = logging.DEBUG  # Could be e.g. "DEBUG" or "WARNING"

logger = logging.getLogger(__name__)
logger.setLevel(LOG_LEVEL)
handler = logging.handlers.TimedRotatingFileHandler(LOG_FILENAME, when="midnight", backupCount=3)
formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

class MyLogger(object):
        def __init__(self, logger, level):
                """Needs a logger and a logger level."""
                self.logger = logger
                self.level = level

        def write(self, message):
                if message.rstrip() != "":
                        self.logger.log(self.level, message.rstrip())

#sys.stdout = MyLogger(logger, logging.INFO)
#sys.stderr = MyLogger(logger, logging.ERROR)

logger.info("*** Starting PiScrape.Py ***")
#logger.info("*** FORCED SYSTEM EXIT ***")
#sys.exit()

port = config.get("system", "serialport") 
baudrate = config.get("system", "baudrate") 

serTimeout = float(.005) 
TGIDold = 0
metadata = ''

try:
    serialFromScanner = serial.Serial(port, baudrate, timeout=serTimeout)
    serialFromScanner.bytesize = serial.EIGHTBITS 
    serialFromScanner.parity = serial.PARITY_NONE 
    serialFromScanner.stopbits = serial.STOPBITS_ONE 
    serialFromScanner.timeout = 1          
    serialFromScanner.xonxoff = False     
    serialFromScanner.dsrdtr = False
except:
    logger.error('*** Failed to open serial port ***', exc_info=True)
serialFromScanner.flushInput()

def getData(test):
    global serBuffer, nextChar, recflag
    global sqlch, parsed
    serBuffer = '' 
    nextChar = ''
    try:
        test = test + '\r\n'
        serialFromScanner.write(test.encode('UTF-8'))
    except:
        logger.error('*** Failed to write data (' + test + ') to serial port ***', exc_info=True)
    time.sleep(.1)

def receiveData():
    if (serialFromScanner.inWaiting() > 0):
        global nextChar, serBuffer
        global sqlch, parsed
        while nextChar != '\r':
            try:
                nextChar = serialFromScanner.read(1).decode('UTF-8')
                serBuffer += nextChar
            except:
                logger.warning('*** read failed ***', exc_info=True)

def setrecflag():
    global recflag, frames, seqnum
    recflag = 0
    seqnum = 1
    frames = []

def parseData(pserBuffer):
    global sqlch, parsed, GLGData, rssi, freq, ft, modu, att, pl, name1, name2, name3
    parsed = pserBuffer.split(",")
    stringtest = parsed[0]
    length = len(parsed) 
    if stringtest == "GLG":
        if (length >= 9):
            try:
                sqlch = parsed[9]
                if (parsed[1] != ""):
                    ft = parsed[1]
                    modu = parsed[2]
                    att = parsed[3]
                    pl = parsed[4]
                    name1 = parsed[5]
                    name2 = parsed[6]
                    name3 = parsed[7]
                    GLGData = parsed 
            except:
                tempstr = '|'.join(parsed)
                logger.warning('*** parse GLG failed  ***' + tempstr, exc_info=True) 
                sqlch = '2'       
    if stringtest == "PWR":
        if (length >= 2):
            try:
                rssi = parsed[1] 
                freq = parsed[2] 
            except:
                tempstr = '|'.join(parsed)
                logger.warning('*** parse PWR failed  ***' + tempstr, exc_info=True) 
                rssi = '0'
                freq = '0000.0000' 

            if (len(rssi) > 0):
                print(rssi + ' - ' + freq)       
                time.sleep(.1)
                serialFromScanner.flushInput()

global sqlch
sqlch = '1'
setrecflag()

#fl = open(config.get("system", "tmpfileloc") + "/seqnum","r")
#fr = fl.read()
#seqnum = int(float(fr))
#fl.close()

r = redis.Redis(
    host='rxwave.com',
    port=6379,
    password='xNfEScQVbqxGCT4eJRa6dmhW2ZSVvHab5Vr2dWVNFWZwnvKr')

seqnum = r.incr('seqnum')

wavfn = config.get("system", "tmpfileloc") + "/" + str(seqnum) + ".wav"

while True: 
    time.sleep(.1)
    getData('GLG')
    receiveData()
    parseData(serBuffer)
    if (sqlch == '0'): 
        # Start recording
        if (recflag == 0):
            cmd = ['sox','-r',config.get("system", "bitrate"),'-c','1','-t','alsa',config.get("system", "audiodevice"),'-t','wav', wavfn] 
            popen = subprocess.Popen ( cmd )
            starttime = datetime.datetime.now()
            logger.debug("*** START recording ***")        
            recflag=1
        getData('PWR')
        receiveData()
        parseData(serBuffer)
    else:
        # Stop recording and save file
        if (recflag == 1): 
            recflag = 0
            stptime = datetime.datetime.now()
            difference = stptime - starttime
            popen.kill()
            try:
                a = config.get("account", "RID")
                b = config.get("account", "hash")
                c = str(seqnum) + ".wav"
                dd = "{0:.2f}".format(difference.microseconds)
                d = str(difference.seconds) + '.' + str(dd)
                d = str(difference.seconds) + '.' + str(difference.microseconds)
                ee = float(freq)/10000
                e = "{0:.4f}".format(ee)         # Freq 0000.0000
                f = name1        # SysSite   || SysSite
                g = name2        # Sys Group || GroupName
                h = name3        # Talk Grp  || ChannelName
                i = "TRNK"        # CallSign  || SysType
                k = ft           # PL
                if (str(ft.find(".")) != "-1"):
                   i = "CONV"        # CallSign  || SysType
                   k = pl           # TalkGroupID

                j = modu         # Modulation
                l = starttime.strftime("%Y-%m-%d %H:%M:%S")  # CutTime StrTime  YYYY-MM-DD HH:MM:SS
                m = rssi         # RSSI from Pidl.py
                n = str(seqnum)  # Internal counter from seq file

                message = a + "~" + b + "~" + c + "~" + d + "~" + e + "~" + f + "~" + g + "~" + h + "~" + i + "~" + j + "~" + k + "~" + l + "~" + m + "~" + n + "~~~~~~~~~"
                print (message)
                logger.debug("Message creation successgful : " + message )
            except:
                a = a
                logger.warning("*** Message creation unsuccessgful "  )


            try:
                connection = pika.BlockingConnection(pika.ConnectionParameters(host='localhost'))
                channel = connection.channel()
                channel.queue_declare(queue='LameQ', durable=True)
                channel.basic_publish(exchange='', routing_key='LameQ', body=message, properties=pika.BasicProperties(delivery_mode = 2,))
                connection.close()
                logger.debug("Message queued to LameQ : " + message )
            except:
                a = a
                logger.warning("*** Message skipped - connect/publish to queue" )

            seqnum = seqnum + 1 
            ft = ""

            fl = open(config.get("system", "tmpfileloc") + "/seqnum","w")
            fl.write(str(seqnum))
            wavfn = config.get("system", "tmpfileloc") + "/" + str(seqnum) + ".wav"
            fl.close()








from setuptools import setup

setup(name='piscan',
      version='0.1.51',
      description='RXWave.com Uniden DMA Police Scanner RaspberryPi agent',
      url='http://www.rxwave.com',
      author='Dave Kolberg',
      author_email='dave.kolberg@gmail.com',
      license='MIT',
      packages=['piscan'],
      install_requires=['redis','pika','boto','flask'],
      zip_safe=False,
      include_package_data=True,
      package_dir={'piscan': 'piscan/data'},
      package_data={
        'data' : ['*'],
      },

)





#!/usr/bin/env python
from __future__ import division
import pika
import sys
import os
import subprocess
import boto
import time
from boto.s3.key import Key
import logging
import logging.handlers
import argparse
import ConfigParser
import string

config = ConfigParser.ConfigParser()
config.read("/etc/piscan/piscan.ini")

#time.sleep(5)

#LOG_FILENAME = config.get("system", "logloc") + "/piqueue.log"
LOG_FILENAME = "/var/log/piqueue.log"
LOG_LEVEL = logging.INFO

logger = logging.getLogger(__name__)
logger.setLevel(LOG_LEVEL)
handler = logging.handlers.TimedRotatingFileHandler(LOG_FILENAME, when="midnight", backupCount=3)
formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(message)s')
handler.setFormatter(formatter)
logger.addHandler(handler)

class MyLogger(object):
        def __init__(self, logger, level):
                """Needs a logger and a logger level."""
                self.logger = logger
                self.level = level

        def write(self, message):
                # Only log if there is a message (not just a new line)
                if message.rstrip() != "":
                        self.logger.log(self.level, message.rstrip())

#sys.stdout = MyLogger(logger, logging.INFO)
#sys.stderr = MyLogger(logger, logging.ERROR)

logger.info("*** Starting PiQueue.Py ***")
connection = pika.BlockingConnection(pika.ConnectionParameters(host='localhost'))
channel = connection.channel()

z = "~"

def SendQueue(message):
        try:
            logger.debug("Connecting to CutQ")
    	    connection1 = pika.BlockingConnection(pika.ConnectionParameters(host='rxwave.com', port=5672, virtual_host='/', credentials=pika.credentials.PlainCredentials('piscan', 'magic4update')))
	    channel = connection1.channel()
	    channel.queue_declare(queue='CutQ', durable=True)
	    channel.basic_publish(exchange='', routing_key='CutQ', body=message, properties=pika.BasicProperties(delivery_mode = 2,))
            logger.info("send message to CutQ - " + message)
	    connection1.close()
        except:
            logger.error("Unable to send message to CutQ")

def digits(x):
    return int(100*x)/100.0

def callback(ch, method, properties, body):
    logger.debug("*** Message Recevied (" + body +") ***")

    try:
        a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w = body.split("~")
        aa = "0"
        try:
            path="/etc/piscan/" + str(n) + ".wav"
            fl=open(path,"rb")
            logger.info("WAV Open : " + str(n) + ".wav" )
            fl.seek(28)
            aa = fl.read(4)
            fl.close()
            byteRate = 0
            for ii in range(4):
                byteRate=byteRate + ord(aa[ii])*pow(256,ii)
            fileSize = os.path.getsize(path)  
            ms = ((fileSize - 44) * 1000) / byteRate
            ctln =  digits(ms/1000)
            aa = str(ctln)
        except:
            logger.error("Unable to determine WAV size")


        logger.debug("Converting WAV")
        subprocess.call(["sudo", "/usr/bin/lame", "-b 32", "-f", "/etc/piscan/" + str(n) + ".wav", "/etc/piscan/" + str(n) + ".mp3"])

        fls = 0
        bb = os.path.getsize("/etc/piscan/" + str(n) + ".mp3")
        logger.debug("MP3 File Size: " + str(fls))

        message="$a~$b~$c~$d~$e$~$f~$g~$h~$i~$j~$k~$l~$m~$n~$o~$p~$q~$r~~~~~~~~~"


        logger.debug("Sending MP3")
        s3_connection = boto.connect_s3(anon=True)
        bucket = s3_connection.get_bucket("cuts",validate=False)
        # try
        key = bucket.new_key('audio/' + str(a) + "_" + str(n) + '.mp3')
        # EXCEPTION HERE
        #     LOGGING HERE
        key.set_contents_from_filename('/etc/piscan/' + str(n) + '.mp3', reduced_redundancy=True)
        #key.set_acl('public-read')
        logger.debug("Send MP3 Successful")

        logger.debug("Sending CutQ message")

        cutqstr = str(a)+z+str(b)+z+str(c)+z+str(d)+z+str(e)+z+str(f)+z+str(g)+z+str(h)+z+str(i)+z+str(j)+z+str(k)+z+str(l)+z+str(m)+z+str(n)+z+str(o)+z+str(p)+z+str(q)+z+str(r)+z+z+z+z+z+z+z+z+z+str(aa)+z+str(bb)
	SendQueue(cutqstr)

        logger.info("Sent: " + cutqstr)

        logger.debug("Deleting MP3")
        os.remove("/etc/piscan/" + str(n) + ".mp3")

    except:
        logger.debug("*** Message from LameQ incorrect size - skipping (" + body +") ***", exc_info=True)

    logger.debug("Deleting WAV")
    try:
        os.remove("/etc/piscan/" + str(n) + ".wav")
    except:
        logger.error("Unable to delete WAV")


    ch.basic_ack(delivery_tag = method.delivery_tag)


logger.debug("Connecting to  queue")
channel.queue_declare(queue='LameQ', durable=True)

logger.debug("Starting Consumption")
channel.basic_qos(prefetch_count=1)
channel.basic_consume(callback, queue='LameQ')
logger.debug("Starting Consumption - 2")
channel.start_consuming()




